package com.linkwechat.wecom.service;

import java.util.List;

import com.baomidou.mybatisplus.extension.service.IService;
import com.linkwechat.wecom.domain.WeTag;
import com.linkwechat.wecom.domain.WeTagGroup;
import com.linkwechat.wecom.domain.dto.WeTagDto;

/**
 * 企业微信标签Service接口
 * 
 * @author ruoyi
 * @date 2020-09-07
 */
public interface IWeTagService  extends IService<WeTag>
{
    /**
     * 查询企业微信标签
     *
     * @param id 企业微信标签ID
     * @return 企业微信标签
     */
    public WeTag selectWeTagById(Long id);

    /**
     * 查询企业微信标签列表
     * 
     * @param weTag 企业微信标签
     * @return 企业微信标签集合
     */
    public List<WeTag> selectWeTagList(WeTag weTag);

    /**
     * 新增企业微信标签
     *
     * @param weTag 企业微信标签
     * @return 结果
     */
    public boolean insertWeTag(WeTag weTag);

    /**
     * 修改企业微信标签
     *
     * @param weTag 企业微信标签
     * @return 结果
     */
    public boolean updateWeTag(WeTag weTag);

    /**
     * 批量删除企业微信标签
     *
     * @param ids 需要删除的企业微信标签ID
     * @return 结果
     */
    public boolean deleteWeTagByIds(List<String> ids);

    /**
     * 删除企业微信标签信息
     *
     * @param id 企业微信标签ID
     * @return 结果
     */
    public boolean deleteWeTagById(String id);



    /**
     * 根据标签id创建标签
     * @param tagId 标签id
     * @return
     */
    public void creatTag(String tagId);

    /**
     * 根据标签id删除标签
     * @param tagId 标签id
     * @return
     */
    public boolean deleteByTagId(String tagId);

    /**
     * 根据标签id修改标签
     * @param tagId 标签id
     * @return
     */
    public void updateTag(String tagId);


    /**
     * 批量保存标签组
     * @param weTagGroups
     * @return
     */
    public void insertWeTagBatch(List<WeTag> weTagGroups);
}
